// System  : MsgInpBx 2.00 - Message/Input Box Library
// File    : TTHERMO.H
// Author  : Eric Woodruff,  CIS ID: 72134,1150
// Updated : Wed 08/02/95 14:37:14
// Note    : Copyright 1992-95, Eric Woodruff, All rights reserved
// Compiler: Borland C++ 3.1 to 4.xx
//
// This file contains the TThermometer class declaration and definitions.
//

#if !defined(__TTHERMO_H)
#define __TTHERMO_H

#if !defined(__FLAT__)

#if !defined(_NEAR)
#define _NEAR   near
#endif

#if !defined(_FAR)
#define _FAR    far
#endif

#endif

#if !defined(ulong)
typedef unsigned long ulong;
#endif

// Thermometer bar option values.
#define thHasTitle 0x0001  // Has a title line
#define thHasMsg   0x0002  // Has a message line
#define thIsBoxed  0x0004  // Thermometer bar is boxed
#define thDoExit   0x0008  // Exit dialog when thermFunc() is finished
#define thMsgAbove 0x0010  // Put message line above bar (implies thHasMsg).

// Combinations that affect height.
#define thTtlMsg    0x0003  // Bar with title and message line
#define thTtlBox    0x0005  // Bar with title and box
#define thMsgBox    0x0006  // Bar with message line and box
#define thTtlMsgBox 0x0007  // Bar with title, message line, and box

// Bit flags used internally
#define thDispMsg  0x0020  // Tells draw() when a message is to be drawn
#define thDispTtl  0x0040  // Tells draw() when a title is to be drawn

// Thermometer bar filler types.
// Most can be used as either a background or foreground.
#define pbfSolid        '\xDB'
#define pbfEmpty        '\x20'
#define pbfTinyDots     '\xFA'
#define pbfSmallDots    '\xF9'
#define pbfBigDots      '\x07'
#define pbfCircle1      '\x08'
#define pbfCircle2      '\x0A'
#define pbfDashedLine1  '\x16'
#define pbfDashedLine2  '\x2D'
#define pbfDashedLine3  '\xF0'
#define pbfDashedLine4  '\xFE'
#define pbfArrowHeads   '\xAF'
#define pbfLightXHatch  '\xB0'      // Light crosshatch.
#define pbfMedXHatch    '\xB1'      // Medium crosshatch.
#define pbfDarkXHatch   '\xB2'      // Dark crosshatch.
#define pbfThinLine     '\xC4'
#define pbfDoubleLine   '\xCD'
#define pbfThickLine1   '\xDC'
#define pbfThickLine2   '\xDF'

// This is a pointer to the attached thermometer bar process.
typedef void (*thermProcess)(void *Owner, void *thBar);

// Thermometer bar data structure for getData()/setData().
// If you use this data structure to modify THOptions, just be sure that
// the TRect() you specify in the constructor has enough rows to handle
// any box, title, or message line that gets enabled.
struct ThermoDataRec {
    thermProcess thermFunc;     // This is always set.

    Boolean KeepCurText;    // If true, neither of these
    char    Title[256];     // will be altered by setData().
    char    Message[256];

    Boolean KeepCurVals;    // If true, neither of these will be altered
    ulong   totIter;        // by setData().
    ulong   curIter;

    Boolean KeepCtorDefs;   // If true, the constructor defaults are
    ushort  THOptions;      // retained and none of these will be
    uchar   backChar;       // altered by setData().
    uchar   foreChar;
    ushort  thermCmd;
};

// ****************************************************************************
// TThermometer bar class declaration.
class TThermometer : public TView
{
private:
    thermProcess  thermFunc;    // Pointer to the process function.
    ushort  thermCmd;           // Thermometer process activation command

    char    Title[256];     // Title line text
    char    Message[256];   // Message line text

    ushort  THOptions;      // Thermometer bar viewing options.
    uchar   backChar;       // Background character
    uchar   foreChar;       // Foreground (percentage done) character

    ulong   totIter;        // Total iterations to perform
    ulong   curIter;        // Current iteration
    short   barWidth;       // Width of the percentage bar

    short   hOffset;        // Horizontal bar offset
    short   vOffset;        // Vertical bar offset
    short   bOffset;        // Box border vertical offset
    short   mOffset;        // Message line vertical offset

public:
    // Has defaults for everything but TRect &r.
    TThermometer(const TRect &r, thermProcess thFunc = NULL, ulong TotIts = 0L,
        ushort Opts = thIsBoxed, uchar backGrnd = pbfLightXHatch,
        uchar foreGrnd = pbfSolid, ushort ActCmd = cmOK);

    virtual void   handleEvent(TEvent& event);
    virtual ushort dataSize(void);          // Get size of thermometer parms.
    virtual void   getData(void *rec);      // Get thermometer parameters.
    virtual void   setData(void *rec);      // Set thermometer parameters.

    virtual void drawBox(void);     // Draw the box around the bar.
    virtual void draw(void);        // Draw the title, bar, and message.

    virtual void cleanUp(void);               // Clean-up but don't destroy.

    inline void setTotIter(ulong newTot)  // Set the total iteration value
        { totIter = newTot;               // and adjust the display.
          dispCurIter(curIter); }

    inline ulong getTotIter(void)         // Get the total iteration value.
        { return totIter; }

    inline ulong getCurIter(void)         // Get the current iteration.
        { return curIter; }

    inline double percentDone(void)       // Get percentage completed.
    {
        return (!totIter) ? (double)0.0 :        // Don't divide by zero!
                     (double)curIter / (double)totIter * (double)100.0;
    }

    inline const char *getMsgTxt(void)    // Get the current message text.
        { return Message; }

    inline const char *getTtlTxt(void)    // Get the current title text.
        { return Title; }

    void setMessage(const char *fmt, ...);  // Format message line.
    void setTitle(const char *fmt, ...);    // Format title line.

    void dispCurIter(ulong newIter);      // Set the current iteration value
                                          // and display the percentage bar.
// This stuff makes it streamable.
private:
    virtual const char *streamableName() const
        { return name; }

protected:
    TThermometer( StreamableInit ) : TView( streamableInit ) {};

public:
    static const char * const _NEAR name;
    static TStreamable *build();

protected:
    virtual void write( opstream& );
    virtual void *read( ipstream& );
};

inline ipstream& operator >> ( ipstream& is, TThermometer& cl )
    { return is >> (TStreamable&)cl; }
inline ipstream& operator >> ( ipstream& is, TThermometer*& cl )
    { return is >> (void *&)cl; }

inline opstream& operator << ( opstream& os, TThermometer& cl )
    { return os << (TStreamable&)cl; }
inline opstream& operator << ( opstream& os, TThermometer* cl )
    { return os << (TStreamable *)cl; }

#endif      // __TTHERMO_H
